
/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * debug section: 4 (reading filters)
 */

/* libowfat */
#include "openreadclose.h"

/* squidwall */
#include "squidwall.h"

#define STATE_NONE 0 /* no file in filter-definition */
#define STATE_SOME 1 /* some files */
#define STATE_ALL  2 /* all files need to be read! */

void read_filters_file(struct squidwall_acl *acl, const char *path, const char *file);
void read_filters_acl(struct squidwall_acl *acl, const char *path, const char *file);

static void read_filters_addfile(stralloc *sa, int *state, char *start);
static void read_filters(request_t *r, struct squidwall_acl *acl, stralloc *cont);

static void read_filters_addfile(stralloc *sa, int *state, char *start)
{
	char *x=start;
	int i, l;

	if (!x) return;
	if (!x[1]) return;
	if (x[0] != ',') return;

	i = 0;
	while (*++x) {

		/* read all available filters and return */
		if (*x == '*') {
			*state = STATE_ALL;
			return;
		}

		/* ',' -> '\0' */
		if (*x == ',') *x = 0;
	}

	/* read each given filter */
	l = str_len(start);
	for (x=start+1; i < l; i+=str_len(x+i)+1) {
		if (!stralloc_cats(sa, x+i)) die_nomem();
		if (!stralloc_0(sa)) die_nomem();
		*state = STATE_SOME;
	}

	return;
}

/**
 * read_filters - read filter definitions from some file
 *
 * @param acl:  the access control list
 * @param path: the acl path
 * @param cont: content of the filter (null terminated lines)
 */
static void read_filters(request_t *r, struct squidwall_acl *acl, stralloc *cont)
{
	//struct squidwall_phrase *phrase;
	//struct squidwall_string *string;
	//struct squidwall_regex *regex;
	//struct list_head *l;
	unsigned int u;
	int todo_done;
	stralloc sa;

	stralloc_init(&sa);

	/* nothing was read currently */
	todo_done=TODO_NONE;
	acl->todo=TODO_DEFAULT;

	/* what is wanted */
	for (u=0; u<cont->len; u+=str_len(cont->s)+1) {
		char *x = cont->s + u;
		int len;

		/* these lines are ignored */
		len = str_len(x);
		if (len < 2) continue;

		switch (*x) {
		case '+':
			x++;
			switch (*x) {

			/* url lists */
			case 'b':
				break;
			case 'u':
				break;
			case 'w':
				break;

			/* content filtering */
			case 'a':
				break;
			case 'c':
				break;
			case 'C':
				break;
			case 'p':
				break;
			case 'V':
				break;
			} /* switch sa.s[1] */
			break;

		case '-':
			x++;
			switch (*x) {

			/* url lists */
			case 'b':
				break;
			case 'u':
				break;
			case 'w':
				break;

			/* content filtering */
			case 'a':
				break;
			case 'c':
				break;
			case 'C':
				break;
			case 'p':
				break;
			case 'V':
				break;
			} /* switch sa.s[1] */
			break;
		} /* switch sa.s[0] */
	}

	debug(4,7, "read_filters(), acl->todo=", acl->todo);
	stralloc_free(&sa);

	return;
}


/**
 * read_filters_file - read filter definitions from a file
 *
 * @param acl   the acl, to be filled
 * @param path  path to the acl's
 * @param file  the filename with the acl
 * @return      nothing
 */
void read_filters_file(struct squidwall_acl *acl, const char *path, const char *file)
{
	stralloc fn, cont;
	unsigned int u;

	stralloc_init(&fn);
	stralloc_init(&cont);

	debug(4,7, "read_filters_file(%s) start", file);

	if (!stralloc_copys(&fn, path)) die_nomem();
	if (!stralloc_cats(&fn, "/")) die_nomem();
	if (!stralloc_cats(&fn, file)) die_nomem();
	fn.s[fn.len]=0;
	
	if (openreadclose(fn.s, &cont, BUFSIZE) == -1) {
		stralloc_free(&fn);
		stralloc_free(&cont);
		debug(4,7, "read_filters_file(%s) eof (BAD)", file);
		return;
	}

	/* terminate all lines */
	for (u=0; u<cont.len; u++)
		if (cont.s[u] == '\n') cont.s[u]=0;

	read_filters(r, acl, &cont);

	debug(4,7, "read_filters_file(%s) eof", file);

	stralloc_free(&fn);
	stralloc_free(&cont);

	return;
}

/**
 * read_filters_acl - read filter definitions from some user/host acl
 *
 * @param acl   the acl, to be filled
 * @param path  path to the acl's
 * @param file  username, host or ip
 * @return      nothing
 */
void read_filters_acl(struct squidwall_acl *acl, const char *path, const char *file)
{
	stralloc fn;
	
	return;

	stralloc_init(&fn);
	
	debug(4,7, "read_filters_acl() start");

	/* read defaults */
	if (!stralloc_copys(&fn, path)) die_nomem();
	if (!stralloc_cats(&fn, "/!")) die_nomem();
	fn.s[fn.len]=0;
	read_filters_file(acl, path, file);

	/* read special stuff */
	if (!stralloc_copys(&fn, "/?")) die_nomem();
	if (!stralloc_cats(&fn, file)) die_nomem();
	fn.s[fn.len]=0;
	read_filters_file(acl, path, file);

	debug(4,7, "read_filters_acl() eof");

	stralloc_free(&fn);

	return;
}

